@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.4 2005/09/02 14:28:20 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.5 2005/11/09 14:12:21 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -57,6 +57,7 @@ static int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp);
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend);
 static void cancel_request(CancelPacket *sp, int secondary_backend);
 static RETSIGTYPE die(int sig);
+static RETSIGTYPE close_idle_connection(int sig);
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 static void send_frontend_exits(void);
 
@@ -86,6 +87,7 @@ void do_child(int unix_fd, int inet_fd)
 	signal(SIGALRM, SIG_DFL);
 	signal(SIGTERM, die);
 	signal(SIGINT, die);
+	signal(SIGHUP, close_idle_connection);
 	signal(SIGQUIT, die);
 	signal(SIGCHLD, SIG_DFL);
 	signal(SIGUSR1, SIG_DFL);
@@ -843,6 +845,7 @@ static RETSIGTYPE die(int sig)
 				pool_debug("child receives smart shutdown request but it's not in idle state");
 				return;
 			}
+			break;
 
 		case SIGINT:	/* fast shutdown */
 		case SIGQUIT:	/* immediate shutdown */
@@ -857,6 +860,37 @@ static RETSIGTYPE die(int sig)
 	exit(0);
 }
 
+static RETSIGTYPE close_idle_connection(int sig)
+{
+	int i;
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+
+	pool_debug("child receives close connection request");
+
+	for (i=0;i<pool_config.max_pool;i++, p++)
+	{
+		if (!MASTER_CONNECTION(p))
+			continue;
+		if (MASTER_CONNECTION(p)->sp->user == NULL)
+			continue;
+
+		if (MASTER_CONNECTION(p)->closetime > 0)		/* idle connection? */
+		{
+			pool_debug("close_idle_connection: close idle connection: user %s database %s", MASTER_CONNECTION(p)->sp->user, MASTER_CONNECTION(p)->sp->database);
+			pool_send_frontend_exits(p);
+			pool_free_startup_packet(MASTER_CONNECTION(p)->sp);
+			pool_close(MASTER_CONNECTION(p)->con);
+
+			if (DUAL_MODE)
+			{
+				/* do not free memory! we did not allocate them */
+				pool_close(SECONDARY_CONNECTION(p)->con);
+			}
+			memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+		}
+	}
+}
+
 static void send_frontend_exits(void)
 {
 	int i;
