@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/main.c,v 1.8 2005/09/02 14:24:54 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/main.c,v 1.9 2005/11/09 14:12:21 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -278,6 +278,7 @@ int main(int argc, char **argv)
 	pool_signal(SIGCHLD, reap_handler);
 	pool_signal(SIGUSR1, failover_handler);
 	pool_signal(SIGUSR2, failover_handler);
+	pool_signal(SIGHUP, exit_handler);
 
 	pool_log("pgpool successfully started");
 
@@ -687,6 +688,8 @@ static RETSIGTYPE exit_handler(int sig)
 		pool_log("received fast shutdown request");
 	else if (sig == SIGQUIT)
 		pool_log("received immediate shutdown request");
+	else if (sig == SIGHUP)
+		pool_log("received idle connection close request");
 	else
 	{
 		pool_error("exit_handler: unknown signal received %d", sig);
@@ -705,6 +708,13 @@ static RETSIGTYPE exit_handler(int sig)
 		}
 	}
 
+	if (sig == SIGHUP)
+	{
+		exiting = 0;
+		POOL_SETMASK(&UnBlockSig);
+		return;
+	}
+
 	POOL_SETMASK(&UnBlockSig);
 
 	while (wait(NULL) > 0)
