@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.7 2005/10/22 12:57:57 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.8 2005/11/09 14:12:21 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -107,6 +107,7 @@ static int replication_was_enabled;		/* replication mode was enabled */
 static int master_slave_was_enabled;	/* master/slave mode was enabled */
 static int internal_transaction_started;		/* to issue table lock command a transaction
 												   has been started internally */
+static int is_drop_database(char *query);		/* returns non 0 if this is a DROP DATABASE command */
 
 POOL_STATUS pool_process_query(POOL_CONNECTION *frontend, 
 							   POOL_CONNECTION_POOL *backend,
@@ -526,6 +527,29 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 
 	pool_debug("Query: %s", string);
 
+	/*
+	 * if this is DROP DATABASE command, send HUP signal to parent and
+	 * ask it to close all idle connections.
+	 * XXX This is overkill. It would be better to close the idle
+	 * connection for the database which DROP DATABASE command tries
+	 * to drop. This is impossible at this point, since we have no way
+	 * to pass such info to other processes.
+	 */
+	if (is_drop_database(string))
+	{
+		int stime = 5;	/* XXX give arbitary time to allow closing idle connections */
+
+		pool_debug("Query: sending HUP signal to parent");
+
+		kill(getppid(), SIGHUP);		/* send HUP signal to parent */
+
+		/* we need to loop over here since we will get HUP signal while sleeping */
+		while (stime > 0)
+		{
+			stime = sleep(stime);
+		}
+	}
+
 	/* process status reporting? */
 	if (strncasecmp(sq, string, strlen(sq)) == 0)
 	{
@@ -1996,7 +2020,7 @@ void pool_send_frontend_exits(POOL_CONNECTION_POOL *backend)
 	 * XXX we cannot call pool_flush() here since backend may already
 	 * close the socket and pool_flush() automatically invokes fail
 	 * over handler. This could happen in copy command (remember the
-	 * famouse "lostsynchronization with server, resettin g
+	 * famouse "lost synchronization with server, resettin g
 	 * connection" message)
 	 */
 	pool_flush_it(MASTER(backend));
@@ -2573,6 +2597,38 @@ static char *get_insert_command_table_name(char *query)
 	return table;
 }
 
+/* judge if this is a DROP DATABASE command */
+static int is_drop_database(char *query)
+{
+	/* skip comment */
+    query = skip_comment(query);
+
+	if (*query == '\0')
+		return 0;
+
+	/* skip spaces */
+	while (*query && isspace(*query))
+		query++;
+
+	/* DROP? */
+	if (strncasecmp("DROP", query, 4))
+		return 0;
+
+	/* skip DROP */
+	while (*query && !isspace(*query))
+		query++;
+
+	/* skip spaces */
+	while (*query && isspace(*query))
+		query++;
+
+	/* DATABASE? */
+	if (strncasecmp("DATABASE", query, 8))
+		return 0;
+
+	return 1;
+}
+
 /* skip SQL comments */
 static char *skip_comment(char *query)
 {
